using System;
using System.Collections.Generic;
using System.Text;

namespace DoxygenWrapper.Wrappers.Compounds.Types
{
	public enum CompoundTypeModifers
	{
		None,
		Unsigned,
		Signed,
		Const,
		Inline,
		Typename,
		Struct,
		Union,
		Pointer,
		Reference
	}
}
