using System;

namespace MyGUI.Sharp
{
    public enum MouseButton : int
    {
        None = -1,
        Left = 0,
        Button0 = 0,
        Button1 = 1,
        Right = 1,
        Button2 = 2,
        Middle = 2,
        Button3 = 3,
        Button4 = 4,
        Button5 = 5,
        Button6 = 6,
        Button7 = 7,
    }
}
