//InsertPoint
#include "MyGUI_Managed_MultiListItem.h"
#include "MyGUI_Managed_TabItem.h"
#include "MyGUI_Managed_MenuItem.h"
#include "MyGUI_Managed_Window.h"
#include "MyGUI_Managed_Widget.h"
#include "MyGUI_Managed_ScrollBar.h"
#include "MyGUI_Managed_TabControl.h"
#include "MyGUI_Managed_TextBox.h"
#include "MyGUI_Managed_ImageBox.h"
#include "MyGUI_Managed_ScrollView.h"
#include "MyGUI_Managed_ProgressBar.h"
#include "MyGUI_Managed_PopupMenu.h"
#include "MyGUI_Managed_MultiListBox.h"
#include "MyGUI_Managed_MenuControl.h"
#include "MyGUI_Managed_MenuBar.h"
#include "MyGUI_Managed_ListBox.h"
#include "MyGUI_Managed_ItemBox.h"
#include "MyGUI_Managed_EditBox.h"
#include "MyGUI_Managed_DDContainer.h"
#include "MyGUI_Managed_ComboBox.h"
#include "MyGUI_Managed_Button.h"
