
#pragma once

namespace msdfgen {

/// Edge color specifies which color channels an edge belongs to.
enum EdgeColor {
    BLACK = 0,
    RED = 1,
    GREEN = 2,
    YELLOW = 3,
    BLUE = 4,
    MAGENTA = 5,
    CYAN = 6,
    WHITE = 7
};

}
