using System;

namespace MyGUI.Sharp
{
    public enum Align : int
    {
        Center = 0,
        VCenter = 0,
        HCenter = 0,
        Left = 2,
        Right = 4,
        HStretch = 6,
        Top = 8,
        Default = 10,
        Bottom = 16,
        VStretch = 24,
        Stretch = 30,
    }
}
