using System;

namespace TestApp.Sharp
{
	public class Test
	{
		public static void TestWidgets()
		{
			Test_Button.Test();
			Test_ComboBox.Test();
			Test_DDContainer.Test();
			Test_EditBox.Test();
			Test_ItemBox.Test();
			Test_ListBox.Test();
			Test_MenuBar.Test();
			Test_MenuControl.Test();
			Test_MultiListBox.Test();
			Test_PopupMenu.Test();
			Test_ProgressBar.Test();
			Test_ScrollView.Test();
			Test_ImageBox.Test();
			Test_TextBox.Test();
			Test_TabControl.Test();
			Test_ScrollBar.Test();
			Test_Widget.Test();
			Test_Window.Test();

			Test_Gui.Test();
		}
	}
}
