/*
 * This source file is part of MyGUI. For the latest info, see http://mygui.info/
 * Distributed under the MIT License
 * (See accompanying file COPYING.MIT or copy at http://opensource.org/licenses/MIT)
 */

#include "MyGUI_Precompiled.h"
#include "MyGUI_Constants.h"

namespace MyGUI
{

	UString Constants::mUString;
	std::string Constants::mString;
	IntSize Constants::mIntSize;
	IntPoint Constants::mIntPoint;

} // namespace MyGUI
